{ ******************************************************************************
{
{                                 508FrameWork
{
{ The framework loops through all the components on a form and identifies which
{ could potentially be a 508 issue. Once identified, a component is created to
{ monitor this component. As it's enabled property (at this time) is switched
{ The component will fire off the staticlabel so that a screen reader may focus
{ and alert the user.
{
{ During the loop the control is compared to the VA508PotentialClasses array.
{ If the control is found then it is identified as a potential 508 issue and
{ the 508 monitor component is created.
{
{ If the control is identified as a container then a monitor is created to
{ watch for any dynamic controls that may be created and then try to identify
{ them as potential 508 items or not.
{
{ This can either be ran at the application level by calling
{ PreLoad508Fix or this can be ran for a specific form by passing in the form
{ as the parameter. When adding new items, please add them to the
{ VA508PotentialClasses array and handle them in HandleOurLabel.
{
{ Conditional Defines
{ AUTO
{     If the AUTO conditional define is set then when looping through the
{     controls the VA508PotentialClasses array is ignored. The framework will
{     attempt to auto identify potential 508 issues by disabling (if necessary)
{     the control and setting focus. If this fails then the control is treated
{     as a potential 508 item and a monitor component is created.
{
{ 508Report
{     If the 508Report conditional define is set then as components are added
{     or ignored, an XML file is created in the Appdata\local\Application Name
{     folder. This provides vital information on which controls are being added
{     and ignored. If not using the AUTO condition, this can identify controls
{     that may need to be added to the VA508PotentialClasses array. If you do
{     not want controls to be reported in the ignored list, then add them to the
{     VA508DNRClasses array.
{
{
{
{
{ Written by: Chris Bell
{ Created: (03/11/14)
{
{
{ Last modified by: David Salliotte
{
{
{
{
{
{ DONE -oChris Bell : Unit Description }
{ DONE -oChris Bell : Form Dynamic Creation }
{ DONE -oChris Bell : Test Total Counts }
{ TODO -oChris Bell : Add ScreenReaderActive code into this unit }
{ DONE -oChris Bell : Rename IgnoreList to DoNotReport list }
{ ****************************************************************************** }

unit U_508FrameWork;

interface

uses Controls, Forms, Dialogs, StdCtrls, Messages,
  Classes, StrUtils, SysUtils, Windows, SHFolder, XMLIntf,
  XMLDoc, Buttons;

type
  // Hook for the component needing a 508 fix
  t508GlobalHook = class(TComponent)
  private
    FOldWndProc: TWndMethod; // Holds the original Windows Procedure
    OurControl: TwinControl; // The control that is note compliant
    OrigCaption: String; // The original caption
    Origwidth: Integer; // The original width
    VA508Label: TStaticText; // The label used to meet 508 standards
    procedure WndProc(var Msg: TMessage); // The new windows procedure
    procedure HandleOurLabel(); // Handle all the labels
  end;

  // Hook into any containers that might have non 508 compliant issues
  tDynamicContianerMonitor = class(TComponent)
  private
    FOldWndProc: TWndMethod; // Holds the original Windows Procedure
    procedure WndProc(var Msg: TMessage); // The new windows procedure
  end;

  // Hooks into the application and monitore the for Screen Readers
  tScreenReaderMonitor = class(TComponent)
  private
    FOldWndProc: TWndMethod; // Holds the original Windows Procedure
    fSRActive: Boolean;
    fEdit: TEdit;
    function  AppHook(var Msg: TMessage): Boolean; // The new windows procedure
    function  AppHook2(var Msg: TMessage): Boolean; // The new windows procedure
    procedure WndProc(var Msg: TMessage); // The new windows procedure
    function GetScreenReader(): Boolean;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  public
    property Active: Boolean read GetScreenReader;
  end;

  TWindowHook = function (var Message: TMessage): Boolean of object;

procedure PreLoad508Fix(FormToLoad: TCustomForm = nil);
// Loop through and load up potential 508 issues
procedure Load508Items(Control: TComponent; SubNode: String = '');
// Loops through and loads 508 controls for a form

procedure AddToXMLLog(TextToAdd, AddToNode: string; IncCount: Boolean = false);
// Creates a log of potential issues
function CheckForPossibleErrors(TheControl: TComponent): String;
// checks a control for possible 508 label issues
function GetXMLFileName(): String;

function Piece(const S: string; Delim: char; PieceNum: Integer): string;
function CountOccurences(const SubText: string; const Text: string): Integer;


Var
 ScreenReaderMonitor: tScreenReaderMonitor;

// can be used to only load certain classes
const
  VA508PotentialClasses: array [0 .. 4] of string = ('TEdit', 'TComboBox',
    'TCheckBox', 'TButton', 'TBitBtn');
  VA508DNRClasses: array [0..3] of string = ('TLabel', 'TStaticLabel', 'TStaticText', 'TBevel');

implementation

{*******
  tScreenReaderMonitor Class
********}

function  tScreenReaderMonitor.AppHook(var Msg: TMessage): Boolean;
begin
  if Msg.Msg = wm_getobject then
   fSRActive := True;
  if Msg.Msg = wm_settingchange then
   fSRActive := false;
end;

function  tScreenReaderMonitor.AppHook2(var Msg: TMessage): Boolean;
begin
  if Msg.Msg = wm_getobject then
   fSRActive := True;
  if Msg.Msg = wm_settingchange then
   fSRActive := false;

end;

function tScreenReaderMonitor.GetScreenReader(): Boolean;
begin
  if not Assigned(ScreenReaderMonitor) then
   ScreenReaderMonitor := tScreenReaderMonitor.Create(Application);
 // ScreenReaderMonitor.fEdit.SetFocus;
  Result := ScreenReaderMonitor.fSRActive;
end;

constructor tScreenReaderMonitor.Create(AOwner: TComponent);
var
 Hook: TWindowHook;
begin
 inherited Create(AOwner);
 Application.HookMainWindow(AppHook);
 Hook := AppHook2;
 // fEdit := TEdit.CreateParented(HWND_MESSAGE);
fEdit := TEdit.Create(Application);
fEdit.Parent := Application.MainForm;
// fEdit.Visible := false;
// FOldWndProc := fEdit.WindowProc;
// fEdit.WindowProc := WndProc;
 SendMessage(fEdit.Handle, CM_WINDOWHOOK, 0,  Windows.LPARAM(@@Hook));

end;

destructor tScreenReaderMonitor.Destroy;
begin
  inherited;
  Application.UnhookMainWindow(AppHook);
end;

procedure tScreenReaderMonitor.WndProc(var Msg: TMessage);
begin
 if Msg.Msg = wm_getobject then
   fSRActive := True;

 Self.FOldWndProc(Msg);
end;

{*******
  tDynamicContainerMonitor Class
********}

procedure tDynamicContianerMonitor.WndProc(var Msg: TMessage);
var
  Temp: T508GlobalHook;
  VA508Problem, OrigEnable: Boolean;
  ErrStr: String;
begin
  // If there was a control change
  if Msg.Msg = CM_CONTROLCHANGE then
  begin
    // If this is a wincontrol and an insert
    if (TControl(Msg.WParam) is TwinControl) and (Msg.LParam = 1) then
    begin
{$IFNDEF AUTO}
      // If it falls inside of our constants
      if ANSIMatchStr(TControl(Msg.WParam).ClassName, VA508PotentialClasses) then
      begin
        // Possible 508 issue
        Temp := T508GlobalHook.Create(TControl(Msg.WParam).Owner);
        Temp.OurControl := TwinControl(Msg.WParam);
        Temp.FOldWndProc := TwinControl(Msg.WParam).WindowProc;
        TwinControl(Msg.WParam).WindowProc := Temp.WndProc;
{$IFDEF 508Report}
        // Setup for the log
        ErrStr := CheckForPossibleErrors(Temp.OurControl);
        if ErrStr <> '' then begin
          AddToXMLLog(ErrStr, 'Detail^'+Temp.OurControl.Owner.Name + '^Added^' +
            Temp.OurControl.ClassName + '^' + Temp.OurControl.Name + '^Error');
          AddToXMLLog(ErrStr, 'Overall^Error^' + Temp.OurControl.Owner.Name + '^' +
            Temp.OurControl.ClassName + '^' + Temp.OurControl.Name, True)
        end else  begin
          AddToXMLLog('', 'Detail^'+Temp.OurControl.Owner.Name + '^Added^' +
            Temp.OurControl.ClassName + '^' + Temp.OurControl.Name);
          AddToXMLLog('', 'Overall^Added^' + Temp.OurControl.ClassName, True );
        end;
{$ENDIF}
        if not Temp.OurControl.Enabled then
          Temp.HandleOurLabel;
      end
      else
      begin
{$IFDEF 508Report}
        if not ANSIMatchStr(TControl(Msg.WParam).ClassName, VA508DNRClasses)
      then  begin
        // Setup for the log
        AddToXMLLog('', 'Detail^'+TControl(Msg.WParam).Owner.Name + '^Ignored^' +
          TControl(Msg.WParam).ClassName + '^' + TControl(Msg.WParam).Name);
        AddToXMLLog('', 'Overall^Ignored^' + TControl(Msg.WParam).ClassName, True );
      end;
{$ENDIF}
      end;
{$ELSE}
      // Is this tabbable
      if TwinControl(TControl(Msg.WParam)).TabStop then
      begin
        VA508Problem := false;
        OrigEnable := TControl(TControl(Msg.WParam)).Enabled;
        // set this to disabled (if needed)
        If OrigEnable then
          TControl(TControl(Msg.WParam)).Enabled := false;
        try
          try
            // try to set focus
            TwinControl(TControl(Msg.WParam)).SetFocus;
          except
            // This is a problem
            VA508Problem := true;
          end;
        finally
          // re-enable (if needed)
          If OrigEnable then
            TControl(TControl(Msg.WParam)).Enabled := true;
          // if there is a problem then add hook
          if VA508Problem then
          begin
            Temp := T508GlobalHook.Create(TControl(Msg.WParam).Owner);
            Temp.OurControl := TwinControl(TControl(Msg.WParam));
            Temp.FOldWndProc := TwinControl(TControl(Msg.WParam)).WindowProc;
            TwinControl(TControl(Msg.WParam)).WindowProc := Temp.WndProc;
{$IFDEF 508Report}
            // Setup for the log
            ErrStr := CheckForPossibleErrors(TwinControl(Msg.WParam));
            if ErrStr <> '' then begin
              AddToXMLLog(ErrStr, 'Detail^'+TwinControl(Msg.WParam).Name + '^Added^' +
                TwinControl(Msg.WParam).ClassName + '^' +
                TwinControl(Msg.WParam).Name + '^Error');
              AddToXMLLog(ErrStr, 'Overall^Error^' +
              TwinControl(Msg.WParam).Owner.Name  + '^' +
                TwinControl(Msg.WParam).ClassName + '^' +
                TwinControl(Msg.WParam).Name, True );
            end else begin
              AddToXMLLog('', 'Detail^'+TwinControl(Msg.WParam).Name + '^Added^' +
                TwinControl(Msg.WParam).ClassName + '^' +
                TwinControl(Msg.WParam).Name);
              AddToXMLLog('', 'Overall^Added^' +
                TwinControl(Msg.WParam).ClassName, True );
            end;

{$ENDIF}
          end;
        end;
      end;
{$ENDIF}
    end;
  end;
  // Run the original windows procedure
  Self.FOldWndProc(Msg);
end;


{*******
  T508GlobalHook Class
********}
procedure T508GlobalHook.WndProc(var Msg: TMessage);
begin
  // main control moved so align the label correctly
  if Msg.Msg = WM_MOVE then
  begin
    if Assigned(VA508Label) then
    begin
      if OurControl is TCheckBox then
        VA508Label.Left := OurControl.Left + 15
      else
        VA508Label.Left := OurControl.Left;
      VA508Label.Top := OurControl.Top;
    end;
  end;

  if Msg.Msg = cm_enabledchanged then
  begin
    // Handle the labels
    HandleOurLabel;
  end;
  // call the original one
  Self.FOldWndProc(Msg);
end;

procedure T508GlobalHook.HandleOurLabel();
begin
  // Setup for Edit
  if OurControl is TEdit then
  begin
    // Create the label if needed
    if not Assigned(VA508Label) then
    begin
      OrigCaption := TEdit(OurControl).Hint;
      VA508Label := TStaticText.Create(OurControl.Owner);
      VA508Label.Parent := OurControl.Parent;
      VA508Label.Top := OurControl.Top;
      VA508Label.Left := OurControl.Left;
      VA508Label.TabStop := true;
      VA508Label.Caption := OrigCaption + ' disabled';
      VA508Label.TabOrder := OurControl.TabOrder;
    end;
    VA508Label.Visible := not OurControl.Enabled;
  end
  else if (OurControl is TComboBox) then
  begin
    // Create the label if needed
    if not Assigned(VA508Label) then
    begin
      OrigCaption := TComboBox(OurControl).Hint;
      VA508Label := TStaticText.Create(OurControl.Owner);
      VA508Label.Parent := OurControl.Parent;
      VA508Label.Top := OurControl.Top;
      VA508Label.Left := OurControl.Left;
      VA508Label.TabStop := true;
      VA508Label.Caption := OrigCaption + ' disabled';
      VA508Label.TabOrder := OurControl.TabOrder;
    end;
    VA508Label.Visible := not OurControl.Enabled;
  end
  else if OurControl is TCheckBox then
  begin
    if Trim(TCheckBox(OurControl).Caption) <> '' then
    begin
      // Create the label if needed
      if not Assigned(VA508Label) then
      begin
        OrigCaption := TCheckBox(OurControl).Caption;
        Origwidth := OurControl.Width;
        VA508Label := TStaticText.Create(OurControl.Owner);
        VA508Label.Parent := OurControl.Parent;
        VA508Label.Top := OurControl.Top;
        VA508Label.Left := OurControl.Left + 15;
        VA508Label.TabStop := true;
        VA508Label.Caption := OrigCaption + ' disabled';
        VA508Label.TabOrder := OurControl.TabOrder;
      end;
      if OurControl.Enabled then
      begin
        OurControl.Width := Origwidth;
        TCheckBox(OurControl).Caption := OrigCaption;
      end
      else
      begin
        OurControl.Width := 20;
        TCheckBox(OurControl).Caption := 'disabled';
      end;
      VA508Label.Visible := not OurControl.Enabled;
    end;
  end
  else if (OurControl is TButton) or (OurControl is TBitBtn) then
  begin
    // Create the label if needed
    if not Assigned(VA508Label) then
    begin
      OrigCaption := TButton(OurControl).Caption;
      VA508Label := TStaticText.Create(OurControl.Owner);
      VA508Label.Parent := OurControl.Parent;
      VA508Label.Top := OurControl.Top;
      VA508Label.Left := OurControl.Left;
      VA508Label.TabStop := true;
      VA508Label.Caption := OrigCaption + ' button disabled';
      VA508Label.TabOrder := OurControl.TabOrder;
    end;
    VA508Label.Visible := not OurControl.Enabled;
  end;
end;

{*******
  Unit Methods
********}

procedure PreLoad508Fix(FormToLoad: TCustomForm = nil);
var
  I: Integer;
  TheLogFile: String;
  Temp1: tDynamicContianerMonitor;
begin
{$IFDEF 508Report}
  // Setup the log file
  TheLogFile := GetXMLFileName;
  if FileExists(TheLogFile) then DeleteFile(PChar(TheLogFile));
  AddToXMLLog(FormatDateTime('mm/dd/yyyy hh:mm:ss', now), 'RunTime');
{$ENDIF}
  if Assigned(FormToLoad) then
  begin
    Load508Items(FormToLoad);
  end
  else
  begin
    for I := 0 to application.ComponentCount - 1 do
    begin
     Load508Items(application.Components[I]);
    end;
  end;

end;

procedure Load508Items(Control: TComponent; SubNode: String = '');
var
  I: Integer;
  Temp: T508GlobalHook;
  Temp1: tDynamicContianerMonitor;
  VA508Problem, OrigEnable: Boolean;
  ErrStr, FormName: String;
begin
  if Trim(Control.Name) = '' then exit;

  //add monitor for dynamic controls
  Temp1 := tDynamicContianerMonitor.Create(Control);
  Temp1.FOldWndProc := TCustomForm(Control).WindowProc;
  TCustomForm(Control).WindowProc := Temp1.WndProc;

{$IFDEF 508Report}
  // Setup for the log
   FormName := Control.Name;
  AddToXMLLog('Added', 'Detail^'+SubNode+FormName+'^DynamicMonitor');
{$ENDIF}

  // look through all the controls
  for I := 0 to Control.ComponentCount - 1 do
  begin
    if Control.Components[I] is TControl then
    begin
      // If the control is a container then dig into it
      if (csAcceptsControls in TControl(Control.Components[I]).ControlStyle)
      then
      begin
{$IFDEF 508Report}
        // Setup for the log
        AddToXMLLog('', 'Detail^'+FormName+ '^Container^' +
          Control.Components[I].Name);
{$ENDIF}
        Load508Items(Control.Components[I], FormName+ '^Container^');
      end
{$IFNDEF AUTO}
      Else if ANSIMatchStr(Control.Components[I].ClassName, VA508PotentialClasses)
      then
      begin
        // Possible 508 issue
        Temp := T508GlobalHook.Create(application);
        Temp.OurControl := TwinControl(Control.Components[I]);
        Temp.FOldWndProc := TwinControl(Control.Components[I]).WindowProc;
        TwinControl(Control.Components[I]).WindowProc := Temp.WndProc;
{$IFDEF 508Report}
        // Setup for the log
        ErrStr := CheckForPossibleErrors(Control.Components[I]);
        if ErrStr <> '' then begin
          AddToXMLLog(ErrStr, 'Detail^'+FormName+ '^Added^' + Control.Components[I]
            .ClassName + '^' + Control.Components[I].Name + '^Error');
          AddToXMLLog(ErrStr, 'Overall^Error^' + Control.Components[I].Owner.Name + '^' +
          Control.Components[I].ClassName + '^' + Control.Components[I].Name, true);
        end else begin
          AddToXMLLog('', 'Detail^'+FormName+ '^Added^' + Control.Components[I]
            .ClassName + '^' + Control.Components[I].Name);
          AddToXMLLog('', 'Overall^Added^' + Control.Components[I].ClassName, True );
        end;

{$ENDIF}
        if not Temp.OurControl.Enabled then
          Temp.HandleOurLabel;

      end
      else
      begin
{$IFDEF 508Report}
      if not ANSIMatchStr(Control.Components[I].ClassName, VA508DNRClasses)
      then begin
        // Setup for the log
        AddToXMLLog('', 'Detail^'+FormName+ '^Ignored^' + Control.Components[I]
          .ClassName + '^' + Control.Components[I].Name);
        AddToXMLLog('', 'Overall^Ignored^' + Control.Components[I]
          .ClassName, True );
      end;
{$ENDIF}
      end;
{$ELSE}
      else
      begin
        if Control.Components[I] is TControl then
        begin
          // Is this tabbable
          if TwinControl(Control.Components[I]).TabStop then
          begin
            VA508Problem := false;
            OrigEnable := TwinControl(Control.Components[I]).Enabled;
            // set this to disabled (if needed)
            If OrigEnable then
              TControl(Control.Components[I]).Enabled := false;
            try
              try
                // try to set focus
                TwinControl(Control.Components[I]).SetFocus;
              except
                // This is a problem
                VA508Problem := true;
              end;
            finally
              // re-enable (if needed)
              If OrigEnable then
                TControl(Control.Components[I]).Enabled := true;
              // if there is a problem then add hook
              if VA508Problem then
              begin
                Temp := T508GlobalHook.Create(application);
                Temp.OurControl := TwinControl(Control.Components[I]);
                Temp.FOldWndProc := TwinControl(Control.Components[I])
                  .WindowProc;
                TwinControl(Control.Components[I]).WindowProc := Temp.WndProc;
{$IFDEF 508Report}
                // Setup for the log
                ErrStr := CheckForPossibleErrors(Control.Components[I]);
                if ErrStr <> '' then begin
                  AddToXMLLog(ErrStr, 'Detail^'+FormName+ '^Added^' +
                  Control.Components[I].ClassName + '^' +
                  Control.Components[I].Name + '^Error');

                  AddToXMLLog(ErrStr, 'Overall^Error^' + Control.Owner.Name+'^'+
                    Control.Components[I].ClassName + '^' + Control.Components
                    [I].Name, True);
                end else begin
                  AddToXMLLog('', 'Detail^'+Control.Name + '^Added^' + Control.Components
                    [I].ClassName + '^' + Control.Components[I].Name);
                 AddToXMLLog('', 'Overall^Added^' + Control.Components
                    [I].ClassName, True);
                end;
{$ENDIF}
              end;
            end;
          end;
        end;
      end;
{$ENDIF}
    end;
  end;
end;

{*******
  Log Methods
********}

function CheckForPossibleErrors(TheControl: TComponent): String;
begin
  Result := '';
  if TheControl is TwinControl then
  begin
    with TheControl as TwinControl do
    begin
      if (TheControl is TEdit) or (TheControl is TComboBox) then
        if Trim(Hint) = '' then
          Result := 'Hint property missing for: ' + Owner.Name+'.'+Parent.Name+'.'+Name;
    end;
  end;
end;

function GetXMLFileName(): String;
Var
 OurLogFile, LocalOnly, AppDir: string;

   // Finds the users special directory
  function LocalAppDataPath: string;
  const
    SHGFP_TYPE_CURRENT = 0;
  var
    path: array [0 .. MaxChar] of char;
  begin
    SHGetFolderPath(0, CSIDL_LOCAL_APPDATA, 0, SHGFP_TYPE_CURRENT, @path[0]);
    Result := StrPas(path);
  end;

begin
  OurLogFile := LocalAppDataPath;
    if (copy(OurLogFile, length(OurLogFile), 1) <> '\') then
      OurLogFile := OurLogFile + '\';

    LocalOnly := OurLogFile;

    // Now set the application level
    OurLogFile := OurLogFile + application.title;
    if (copy(OurLogFile, length(OurLogFile), 1) <> '\') then
      OurLogFile := OurLogFile + '\';
    AppDir := OurLogFile;

    // try to create or use base direcrtory
    if not DirectoryExists(AppDir) then
      if not ForceDirectories(AppDir) then
        OurLogFile := LocalOnly;

    OurLogFile := OurLogFile +
      '508Framework' { + IntToStr(GetCurrentProcessID) } + '.XML';

   Result := OurLogFile;
end;

procedure AddToXMLLog(TextToAdd, AddToNode: string; IncCount: Boolean = false);
var
  XMLMain: TXMLDocument;
  iRootNode, iNode1, LastNode: IXMLNode;
  I, LoopCnt: Integer;
  TheNode, Cnter: String;
  OurLogFile: string;
begin
  if Trim(AddToNode) = '' then
    exit;

  XMLMain := TXMLDocument.Create(application);
  try
    XMLMain.Active := true;

    OurLogFile := GetXMLFileName;
    If FileExists(OurLogFile) then
    begin
      XMLMain.LoadFromFile(OurLogFile);
      if not Assigned(XMLMain.ChildNodes.FindNode(application.title)) then
        iRootNode := XMLMain.AddChild(application.Name)
      else
        iRootNode := XMLMain.ChildNodes.FindNode(application.title);
    end
    else
      iRootNode := XMLMain.AddChild(application.title);

    iNode1 := nil;
    LastNode := nil;
    LoopCnt := CountOccurences('^', AddToNode);
    for I := 1 to (LoopCnt + 1) do
    begin
      TheNode := Piece(AddToNode, '^', I);
      if Trim(TheNode) = '' then
        TheNode := 'blank' + FormatDateTime('hhmmsszz', now);

      if iNode1 = nil then
        iNode1 := iRootNode.ChildNodes.FindNode(TheNode)
      else
        iNode1 := iNode1.ChildNodes.FindNode(TheNode);

      if not Assigned(LastNode) then
      begin
        if not Assigned(iNode1) then
          iNode1 := iRootNode.AddChild(TheNode);
      end
      else
      begin
        if not Assigned(iNode1) then
          iNode1 := LastNode.AddChild(TheNode);
      end;
      LastNode := iNode1;
    end;
    LastNode := nil;

    if Trim(TextToAdd) <> '' then
      iNode1.Text := TextToAdd;

    if IncCount then begin
       LastNode := iNode1.ParentNode.ChildNodes.FindNode('TotalCount');
       if not Assigned(LastNode) then
          iNode1 := iNode1.ParentNode.AddChild('TotalCount')
       else
        iNode1 := iNode1.ParentNode.ChildNodes.FindNode('TotalCount');

       Cnter := iNode1.text;
       iNode1.Text := IntToStr(StrToIntDef(Cnter, 0) + 1);
    end;

    XMLMain.Xml.Text := XMLDoc.FormatXMLData(XMLMain.Xml.Text);
    XMLMain.Active := true;
    XMLMain.SaveToFile(OurLogFile);
    XMLMain.Active := true;
  finally
    XMLMain.Free;
  end;
end;

{*******
  Utility Methods
********}

function Piece(const S: string; Delim: char; PieceNum: Integer): string;
{ returns the Nth piece (PieceNum) of a string delimited by Delim }
var
  I: Integer;
  Strt, Next: PChar;
begin
  I := 1;
  Strt := PChar(S);
  Next := StrScan(Strt, Delim);
  while (I < PieceNum) and (Next <> nil) do
  begin
    Inc(I);
    Strt := Next + 1;
    Next := StrScan(Strt, Delim);
  end;
  if Next = nil then
    Next := StrEnd(Strt);
  if I < PieceNum then
    Result := ''
  else
    SetString(Result, Strt, Next - Strt);
end;

function CountOccurences(const SubText: string; const Text: string): Integer;
begin
  if (SubText = '') or (Text = '') or (Pos(SubText, Text) = 0) then
    Result := 0
  else
    Result := (length(Text) - length(StringReplace(Text, SubText, '',
      [rfReplaceAll]))) div length(SubText);
end;

end.
